// ESP8266(ESPr Developer)用Scratch対応ファームウエア
// 機能拡張版
// 注意：専用のヘルパーアプリが必要です
// by Takao Takahashi
// https://tsq.gapj.net
// このスケッチはI2Cを使用するためIO4,IO5は使えません

//ESPr Developer のピン定義
//デジタル {12,13,14,15,16};
//アナログ {0}

//mDNS使用(コンパイル時オプション)
#define USEMDNS

//APモード時SoftAP関連設定
// パスワード
#define APPSK  "12345678901234"

//自分のノード名 NetBOIS,mDNSでも使用
//APモード時はこの名前-MAC下3桁がAP名になる
#define MYNODENAME "MYESPR"

//デバッグオンオフ
//デバッグ情報はシリアル出力しますが速度向上のため通常は
//オフして使用してください
//#define DEBUG

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266NetBIOS.h>
#ifdef USEMDNS
#include <ESP8266mDNS.h>
#endif
#include <ESP8266WebServer.h>
#include <Wire.h>
#include <Adafruit_MPL3115A2.h>
//サーボモーターを使用する
#include <Servo.h>

#include <EEPROM.h>

struct WIFICONF {
  uint8_t wifimode;
  char ssid[32];
  char pass[32];
};

char sta_ssid[32];
char sta_pass[32];
const char *password = APPSK;

// Webサーバーのクラス定義
ESP8266WebServer server(80);

// 気温・気圧センサのクラス定義
Adafruit_MPL3115A2 baro = Adafruit_MPL3115A2();

//サーボのクラス定義
Servo myServo1;
Servo myServo2;
Servo myServo3;
Servo myServo4;
Servo myServo5;

// MACアドレス
uint8_t sta_mac_add[6];
uint8_t ap_mac_add[6];
//
char my_node_name[64];

// 気温と気圧保持用
float my_temp;
float my_press;

//各ピン制御値
char pin_cmd;
uint8_t cmd_pin;
uint16_t cmd_value; //ESPrの場合PWMが10bit

// バッファ
char tmp_buf[256];
char sbuffer[32];
//
WIFICONF wifibuf;
unsigned char wifimode;

//タイムスタンプ
unsigned long last_time;

// ルート(/)アクセス時の処理(何もしない)
void handleRoot() {
  server.send(200, "text/html", "<h1>okay</h1>");
}

// ヘルパーからのポーリング処理
// ピン数が限られているので直接読む
void handlePoll() {

  last_time = millis();

  sprintf(tmp_buf, "D12 %d:D13 %d:D14 %d:D15 %d:D16 %d:A0 %d:T %.2f:P %.2f",
    digitalRead(12),
    digitalRead(13),
    digitalRead(14),
    digitalRead(15),
    digitalRead(16),  
    analogRead(0),
    my_temp,my_press);

  server.send(200, "text/html", tmp_buf); 
}

// ポーリングチェック
void checkPoll(){
  unsigned long cur_time;
  uint8_t i;

  cur_time = millis();
  if((cur_time-last_time)>500){
    for(i=12;i++;i<17){
      digitalWrite(i, LOW);
    }
  }
}

// ピン制御(モード、出力)
void handlePinCtrl() {
  String Pmode = server.arg("pmode");
  Pmode.toCharArray(sbuffer, 32);
  if(Pmode != NULL){
    pin_cmd = sbuffer[0];
    cmd_pin = atoi(strtok(sbuffer+1,"-"));
    cmd_value = atoi(strtok(NULL,"-"));
#ifdef DEBUG
    Serial.print("mode : ");
    Serial.print(pin_cmd);
    Serial.print("\n");
    Serial.print("Pin : ");
    Serial.print(cmd_pin);
    Serial.print("\n");
    Serial.print("Value : ");
    Serial.print(cmd_value);
    Serial.print("\n");
#endif
  //ピン値が正常範囲しか処理しない
    if((11<cmd_pin)&&(cmd_pin<17)){
      switch(pin_cmd){
        case 'D': //デジタル出力
          digitalOut();
          break;
        case 'A': //アナログ出力
          analogOut();
          break;
        case 'M': //ピンモード設定
          pinmSet();
          break;
       case 'U': //サーボ設定
         setServo();
         break;
       case 'V': //サーボ角度
         servoAngle();
         break;
       defualt: //該当するコマンドがなければ何もしない
         break;
      }
    }
  }
  server.send(200, "text/html", "OK");
}

//デジタルピン出力(High/Low)
void digitalOut(){
  if(cmd_value == 0){
    digitalWrite(cmd_pin, LOW);
  }
  if(cmd_value == 1){
    digitalWrite(cmd_pin, HIGH);
  }
}

//アナログ(PWM)出力
void analogOut(){
  analogWrite(cmd_pin, cmd_value);
}

//ピンモード設定
//ピンモード設定(INPUT or OUTPUT)
void pinmSet()
{
  if(cmd_value == 0){
    pinMode(cmd_pin, INPUT);
  }
  if(cmd_value == 1){
    pinMode(cmd_pin, OUTPUT);
  }
}

//サーボピン設定
void setServo()
{
  //cmd_pinは11足した値で来るので引く
  cmd_pin = cmd_pin - 11;
  switch(cmd_pin){
    case 1:
      myServo1.attach(cmd_value);
      break;
    case 2:
      myServo2.attach(cmd_value);
      break;
    case 3:
      myServo3.attach(cmd_value);
      break;
    case 4:
      myServo4.attach(cmd_value);
      break;
    case 5:
      myServo5.attach(cmd_value);
      break;
    default:
      break;
  }
}

//サーボ角度設定
void servoAngle()
{
  //cmd_pinは11足した値で来るので引く
  cmd_pin = cmd_pin - 11;
  switch(cmd_pin){
    case 1:
      myServo1.write(cmd_value);
      break;
    case 2:
      myServo2.write(cmd_value);
      break;
    case 3:
      myServo3.write(cmd_value);
      break;
    case 4:
      myServo4.write(cmd_value);
      break;
    case 5:
      myServo5.write(cmd_value);
      break;
    default:
      break;
  }
}

// 気温・気圧取得処理
void handleTempPress(){
  if(! baro.begin() ){
    my_press = float(0);
    my_temp = float(0);
  } else {
    my_press = baro.getPressure();
    //PaからhPaに変換
    my_press = my_press / (float) 100;
    my_temp = baro.getTemperature();
  }
  //デバッグ用
#ifdef DEBUG
  sprintf(tmp_buf, "T:%2.1f P:%4.1f\n", my_temp, my_press);
  Serial.print(tmp_buf);
#endif
  //レスポンス送信
  server.send(200, "text/html", "OK");
}

void setupWifi() {
  uint8_t tmp;

  String wifiParam = server.arg("wifiparam");
  wifiParam.toCharArray(tmp_buf, 96);
  if(wifiParam != NULL){
    tmp = tmp_buf[0] - 0x30;
    strncpy(wifibuf.ssid, strtok(tmp_buf+1,":"), 32);
    strncpy(wifibuf.pass, strtok(NULL, ":"), 32);
#ifdef DEBUG
    Serial.print(wifibuf.ssid);
    Serial.print("\n");
    Serial.print(wifibuf.pass);
    Serial.print("\n");
#endif
    wifibuf.wifimode = tmp;
    EEPROM.put<WIFICONF>(0, wifibuf);
    EEPROM.commit();
    server.send(200, "text/html", "OK");
  } else {
    server.send(200, "text/html", "NG");
  }
}

void setup() {
  uint8_t tmp;
  char t_buf;

  delay(500);
  EEPROM.begin(96);
  wifimode = 0;
  delay(500);
  //デバッグ用にシリアルを使用
  Serial.begin(115200);
  Serial.println();

  Serial.print("Starting.");
  for(tmp=0;tmp<10;tmp++){
    if(Serial.available() > 0){
      t_buf = Serial.read();
      if(t_buf == 'Z'){
        strncpy(wifibuf.ssid, "        ", 32);
        strncpy(wifibuf.pass, "        ", 32);
        wifibuf.wifimode = 0;
        EEPROM.put<WIFICONF>(0, wifibuf);
        EEPROM.commit();
        Serial.print("WiFi Params reseted\n");
      }
    }
    delay(1000);
    Serial.print(".");
  }
  Serial.print("\n");
  
  EEPROM.get<WIFICONF>(0, wifibuf);
  strncpy(sta_ssid, wifibuf.ssid, 32);
  strncpy(sta_pass, wifibuf.pass, 32);
  wifimode = wifibuf.wifimode;
  
  if(wifimode == 1){
    // WiFiステーションモード
    Serial.print("WiFi: STATION\n");
    Serial.print("STA Starting...\n");
    WiFi.mode(WIFI_STA);
    WiFi.begin(sta_ssid, sta_pass);
  } else { // WIFI APモード
    Serial.print("WiFi: ACCESS POINT\n");
    Serial.print("AP Starting...\n");
    // SoftAP起動
    WiFi.softAPmacAddress(ap_mac_add);
    Serial.printf("AP MAC %02X:%02X:%02X:%02X:%02X:%02X\r\n",
      ap_mac_add[0], ap_mac_add[1], ap_mac_add[2], ap_mac_add[3], ap_mac_add[4], ap_mac_add[5]);
    sprintf(my_node_name,"%s-%02x%02x%02x",MYNODENAME,ap_mac_add[3],ap_mac_add[4],ap_mac_add[5]);
    WiFi.softAP(my_node_name, password);
    Serial.print("SSID: ");
    Serial.print(my_node_name);
    Serial.print("\n");
    // WiFi APモード
    IPAddress myIP = WiFi.softAPIP();
    Serial.print("AP IP address: ");
    Serial.println(myIP);
  }

  if(wifimode == 1){
    // Wait for connection
    while (WiFi.status() != WL_CONNECTED) {
      delay(1000);
      Serial.print(".");
    }
    Serial.print("\n");
    Serial.print("Connected to ");
    Serial.println(sta_ssid);
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    WiFi.macAddress(sta_mac_add);
    Serial.printf("STA MAC %02X:%02X:%02X:%02X:%02X:%02X\r\n",
      sta_mac_add[0], sta_mac_add[1], sta_mac_add[2], sta_mac_add[3], sta_mac_add[4], sta_mac_add[5]);
    sprintf(my_node_name,"%s-%02x%02x%02x",MYNODENAME,sta_mac_add[3],sta_mac_add[4],sta_mac_add[5]);
    Serial.print(my_node_name);
    Serial.print("\n");
  }
  
  //各ページのハンドラを登録
  server.on("/", handleRoot);
  server.on("/POLL", handlePoll);
  server.on("/PINCTRL", handlePinCtrl);
  server.on("/TEMPP", handleTempPress);
  server.on("/SETUPWIFIMODE", setupWifi);
  //HTTPサーバを開始
  server.begin();
  Serial.println("HTTP server started");


#ifdef USEMDNS
  if (!MDNS.begin(my_node_name)) {
    Serial.println("Error setting up MDNS responder!");
    while (1) {
      delay(1000);
    }
  }
#endif
  
  NBNS.begin(my_node_name);

  //タイムスタンプ更新
  last_time = millis();
}


//メインループWebサーバで接続待機
void loop() {
#ifdef USEMDNS
  MDNS.update();
#endif
  server.handleClient();
  checkPoll();
}
