// ESP8266(ESPr Developer)用Scratch対応ファームウエア
// 書籍掲載版
// 注意：専用のヘルパーアプリが必要です
// by Takao Takahashi
// https://tsq.gapj.net
// このスケッチはI2Cを使用するためIO4,IO5は使えません

//ESPr Developer のピン定義
//デジタル {12,13,14,15,16};
//アナログ {0}

//WiFiのモード
//WIFISTAが定義されているとステーションモード
//そうでなければAPモード(コンパイル時オプション)
#define WIFISTA

//ステーションモード時設定
//SSID
#define STASSID "YOURSSID"
//パスワード
#define STAPSK "yourpassword1"

//APモード時SoftAP関連設定
// SSID
#define APSSID "MyESPr01"
// パスワード
#define APPSK  "12345678901234"

//NetBIOS名
#define MYNODENAME "MYESPR1"

//デバッグオンオフ
//デバッグ情報はシリアル出力しますが速度向上のため通常は
//オフして使用してください
//#define DEBUG

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266NetBIOS.h>
#include <ESP8266WebServer.h>
#include <Wire.h>
#include <Adafruit_MPL3115A2.h>

#ifdef WIFISTA
const char* ssid = STASSID;
const char* password = STAPSK;
#else
const char *ssid = APSSID;
const char *password = APPSK;
#endif

// Webサーバーのクラス定義
ESP8266WebServer server(80);

// 気温・気圧センサのクラス定義
Adafruit_MPL3115A2 baro = Adafruit_MPL3115A2();

// 気温と気圧保持用
float my_temp;
float my_press;

//各ピン制御値
char pin_cmd;
uint8_t cmd_pin;
uint16_t cmd_value; //ESPrの場合PWMが10bit

// バッファ
char tmp_buf[256];
char sbuffer[32];

// ルート(/)アクセス時の処理(何もしない)
void handleRoot() {
  server.send(200, "text/html", "<h1>okay</h1>");
}

// ヘルパーからのポーリング処理
// ピン数が限られているので直接読む
void handlePoll() {

  sprintf(tmp_buf, "D12 %d:D13 %d:D14 %d:D15 %d:D16 %d:A0 %d:T %.2f:P %.2f",
    digitalRead(12),
    digitalRead(13),
    digitalRead(14),
    digitalRead(15),
    digitalRead(16),  
    analogRead(0),
    my_temp,my_press);

  server.send(200, "text/html", tmp_buf); 
}

// ピン制御(モード、出力)
void handlePinCtrl() {
  String Pmode = server.arg("pmode");
  Pmode.toCharArray(sbuffer, 32);
  if(Pmode != NULL){
    pin_cmd = sbuffer[0];
    cmd_pin = atoi(strtok(sbuffer+1,"-"));
    cmd_value = atoi(strtok(NULL,"-"));
#ifdef DEBUG
    Serial.print("mode : ");
    Serial.print(pin_cmd);
    Serial.print("\n");
    Serial.print("Pin : ");
    Serial.print(cmd_pin);
    Serial.print("\n");
    Serial.print("Value : ");
    Serial.print(cmd_value);
    Serial.print("\n");
#endif
  //ピン値が正常範囲しか処理しない
    if((11<cmd_pin)&&(cmd_pin<17)){
      switch(pin_cmd){
        case 'D': //デジタル出力
          digitalOut();
          break;
        case 'A': //アナログ出力
          analogOut();
          break;
        case 'M': //ピンモード設定
          pinmSet();
          break;
      }
    }
  }
  server.send(200, "text/html", "OK");
}

//デジタルピン出力(High/Low)
void digitalOut(){
  if(cmd_value == 0){
    digitalWrite(cmd_pin, LOW);
  }
  if(cmd_value == 1){
    digitalWrite(cmd_pin, HIGH);
  }
}

//アナログ(PWM)出力
void analogOut(){
  analogWrite(cmd_pin, cmd_value);
}

//ピンモード設定
//ピンモード設定(INPUT or OUTPUT)
void pinmSet()
{
  if(cmd_value == 0){
    pinMode(cmd_pin, INPUT);
  }
  if(cmd_value == 1){
    pinMode(cmd_pin, OUTPUT);
  }
}

// 気温・気圧取得処理
void handleTempPress(){
  if(! baro.begin() ){
    my_press = float(0);
    my_temp = float(0);
  } else {
    my_press = baro.getPressure();
    //PaからhPaに変換
    my_press = my_press / (float) 100;
    my_temp = baro.getTemperature();
  }
  //デバッグ用
#ifdef DEBUG
  sprintf(tmp_buf, "T:%2.1f P:%4.1f\n", my_temp, my_press);
  Serial.print(tmp_buf);
#endif
  //レスポンス送信
  server.send(200, "text/html", "OK");
}

void setup() {
  delay(1000);
  //デバッグ用にシリアルを使用
  Serial.begin(115200);
  Serial.println();
  Serial.print("AP Starting...");

#ifdef WIFISTA
  // WiFiステーションモード
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("");
#else
  // SoftAP起動
  WiFi.softAP(ssid, password);
  // WiFi APモード
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(myIP);
#endif

#ifdef WIFISTA
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
#endif
  
  //各ページのハンドラを登録
  server.on("/", handleRoot);
  server.on("/POLL", handlePoll);
  server.on("/PINCTRL", handlePinCtrl);
  server.on("/TEMPP", handleTempPress);
  //HTTPサーバを開始
  server.begin();
  Serial.println("HTTP server started");

  NBNS.begin(MYNODENAME);
}

//メインループWebサーバで接続待機
void loop() {
  server.handleClient();
}
