// ESPr Developer + Scratch ヘルパー
// このヘルパーに対応したESPrのファームウェアと
// Scratch 拡張機能が必要です
// このjsはNodejsとexpress,serialportが必要です
// by Takao Takahashi
// https://tsq.gapj.net

//--以下の値は使用する環境に応じて変更します

//ESPrのIPアドレスが既知の場合には以下を修正
//APモードで使用している場合にはこの指定方法を使う
//STAモードで使用しIPアドレスがわかる場合にはここで指定する
var BASEURL = 'http://192.168.4.1';

//ESPrをSTAモードで使用してIPアドレスがわからない場合には
//NetBIOS名を指定
const MYNODENAME = 'MYESPR1';
//名前解決を使用しない場合には空文字列を指定
//const MYNODENAME = '';

//
var BASEURI = 'http://';

// Scratch機能拡張用ポート番号
const exten_listen = 8003;


//各種変数
var pinVals = [];
var report_string = "";
var tmp_cnt = 0;

//Scratchからのポーリングに対する応答
var report_string = '';

// expressを使用する
const express = require('express')
      , routes = require('./routes')
      , path = require('path');

const app = express();
const http = require('http');
const dns = require('dns');

//DNS(NetBIOS)で名前が解決できた場合はBASEURLを上書き
if(MYNODENAME != ''){
  dns.lookup(MYNODENAME, (err, address, family) => {
    console.log(address);
    if(address != ''){
      BASEURL = BASEURI + address;
    }
  });
}


//このヘルパーのHTTPでの待機
//Scratch HTTP拡張のため8001で待ち受ける
const server = app.listen(exten_listen, function(){
    console.log('exten listen at.' + server.address().port);
});


//ESPrに対するポーリング処理
//一定間隔(0.1秒)で実行
setInterval(function() {
  //Scratchからのポーリングが高速すぎるため拡張機能とは別個に実行
    //ESPrに対してポーリングを実行
    http.get(BASEURL+'/POLL', (res) => {
      var body = '';
      res.setEncoding('utf8');

      res.on('data', (part_data) => {
        body += part_data;
      });

      res.on('end', (res) => {
        //ESPrからのデータはコロン区切り
        pinVals = body.split(':');
        //Scratchへのリポート文字列を初期化
        report_string = '';
        //各要素について処理
        pinVals.forEach(function(pindata){
          var tmp_str  = pindata.split(' ');
          //リポート文字列は"digitalValue/IO12 1"の形式にして出力
          //頭がDならデジタルピン
          if(tmp_str[0].charAt(0) == 'D'){
            var tmp_pindata = pindata.replace('D', 'IO');
            report_string += 'digitalValue/' + tmp_pindata + '\n';
          }
          //頭がAならアナログピン
          if(tmp_str[0].charAt(0) == 'A'){
            var tmp_data = pindata.split(' ');
            report_string += 'analogValue ' + tmp_data[1] + '\n';
          }
          //頭がTなら温度
          if(tmp_str[0].charAt(0) == 'T'){
            var tmp_data = pindata.split(' ');
            report_string += 'tempreture ' + tmp_data[1] + '\n';
          }
          //頭がPなら気圧
          if(tmp_str[0].charAt(0) == 'P'){
            var tmp_data = pindata.split(' ');
            report_string += 'pressure ' + tmp_data[1] + '\n';
          }
        });
        //console.log(body);
        //console.log(report_string);
      });
    }).on('error', (e) => {
      console.log(e.message);
    });
}, 100); //ポーリング間隔は100mSec


//Scratchからポーリングされた場合に呼ばれる処理
app.get('/poll', (req, res) => {
  //応答するデータはESPrに対するポーリングで生成される
  res.send(report_string);
}).on('error', (e) => {
  console.log(e.message);
});


//Scratchからのリセット要求
app.get('/rest_all', (req, res) => {
  //今のところ何もしない
  console.log('Reset Recieved');
  res.send('OK');
});


//Scratch各ブロックに対応した処理

//ピン値設定(デジタル)
app.get('/pinval/:pinNum/:pinVal', (req, res) => {
  //console.log("Pin " + req.params.pinNum);
  //console.log("To " + req.params.pinVal);
  var tmp_val = '0';
  var tmp_str = req.params.pinNum.replace('IO','D');
  if(req.params.pinVal.toUpperCase() == 'HIGH') tmp_val = '1';
  if(req.params.pinVal.toUpperCase() == 'LOW') tmp_val = '0';
  var tmp_str = tmp_str + '-' + tmp_val;
  var REQURI = BASEURL + '/PINCTRL' + '?' + 'pmode=' + tmp_str;
  http.get(REQURI, (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
});

//PWM出力
app.get('/pinpwm/:pinNum/:pinVal', (req, res) => {
  var tmp_val = req.params.pinVal;
  var tmp_str = req.params.pinNum.replace('IO','A');
  if(Number(tmp_val) > 1023) tmp_val = '1023';
  //console.log("Pin " + tmp_str);
  //console.log("To " + tmp_val);
  if(tmp_val > 1023) tmp_val = '1023';
  tmp_str = tmp_str + '-' + tmp_val;
  var REQURI = BASEURL + '/PINCTRL' + '?' + 'pmode=' + tmp_str;
  http.get(REQURI, (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
});

//ピンモード設定
app.get('/pinmode/:pinNum/:pinMode', (req, res) => {
  //console.log("Pin " + req.params.pinNum);
  //console.log("To " + req.params.pinMode);
  var tmp_val = '0';
  var tmp_str = req.params.pinNum.replace('IO','M');
  if(req.params.pinMode.toUpperCase() == 'OUTPUT') tmp_val = '1';
  if(req.params.pinMode.toUpperCase() == 'INPUT') tmp_val = '0';
  tmp_str = tmp_str + '-' + tmp_val;
  var REQURI = BASEURL + '/PINCTRL' + '?' + 'pmode=' + tmp_str;
  http.get(REQURI, (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
}).on('error', (e) => {
  console.log(e.message);
});

//気温・気圧測定開始
app.get('/gettemppress', (req, res) => {
  http.get(BASEURL+'/TEMPP', (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
});

//サーボピン設定
app.get('/servoPin/:pinNum/:pinVal', (req, res) => {
  var tmp_val = req.params.pinVal.replace('IO','');
  //Hack:ピン値チェックが11～17なので11を足しておく
  var tmp_servo = Number(req.params.pinNum) + Number('11');
  var tmp_str = 'U' + tmp_servo.toString() + '-' + tmp_val;
  var REQURI = BASEURL + '/PINCTRL' + '?' + 'pmode=' + tmp_str;
  http.get(REQURI, (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
}).on('error', (e) => {
  console.log(e.message);
});

//サーボ角度設定
app.get('/servoAngle/:pinNum/:pinVal', (req, res) => {
  //Hack:ピン値チェックが11～17なので11を足しておく
  var tmp_servo = Number(req.params.pinNum) + Number('11');
  var tmp_str = 'V' + tmp_servo.toString() + '-' + req.params.pinVal;
  var REQURI = BASEURL + '/PINCTRL' + '?' + 'pmode=' + tmp_str;
  http.get(REQURI, (rres) => {
    var body = '';
    rres.on('data', (part_data) => {
      body += part_data;
    });
    rres.on('end', (rres) => {
    });
  }).on('error', (e) => {
    console.log(e.message);
  });
  res.send('OK');
}).on('error', (e) => {
  console.log(e.message);
});
